# 一、概述 

HapticPlayer是一套振动波形的生成框架，可以通过参数或者HE格式为你的线性马达机器定制出丰富的振动效果。

# 二、支持设备 

| 注意 |     只支持高压线性马达设备                                   |
| ---- | ------------------------------------------------------------ |
| 小米 | 小米9 pro 5G，小米10全系列（小米10、小米10pro等），小米11全系列（小米11、小米11pro等），小米12全系列（小米12、小米12s、小米12s ultra等)，小米mix4，小米mix fold，红米K系列（红米K30S至尊纪念版、K40全系列、K50全系列)，红米Note系列（红米Note10 pro、红米Note11全系列)|
| OPPO | X70系列X80系列及以后的X系列，S15系列，X Fold， X Note，iQOO 3 5系列 7系列 8系列 9系列 10系列及以后的iQOO数字系列，iQOO Neo5 Neo 6及以后的Neo数字系列                                |
| VIVO | OPPO Reno 6 Pro，OPPO Reno 6 Pro Plus，OPPO Reno 7 Pro，OPPO Reno 8 Pro，OPPO Reno 8 Pro Plus，OPPO Find X5，OPPO Find X5 Pro，1+ 9 R，1+ 9 RT，1+ 10 Pro，1+ Ace，1+ Ace Pro                                |

# 三、API接口

#### android.os.DynamicEffect

| 接口                              | 说明                                                         |
| --------------------------------- | ------------------------------------------------------------ |
| DynamicEffect create(String json) | 系统实现一种扩展的振动效果android.os.DynamicEffect。其中create为静态方法，参数为振动效果描述的json字符串，用来创建DynamicEffect。 |

#### android.os.HapticPlayer

HapticPlayer为振动接口的封装（注：带\*的接口系统侧可能会不支持，调用后系统侧不会进行处理）

| 接口                                                         | 说明                                                         |
| ------------------------------------------------------------ | ------------------------------------------------------------ |
| HapticPlayer (DynamicEffect effect)                          | 构造方法，effect为创建的振动效果DynamicEffect                |
| boolean isAvailable()                                        | 判断系统是否支持                                             |
| void start(int loop)                                         | 开始播放效果@param： loop 循环次数, 1不循环，大于1循环次数，-1无限循环； |
| void start(int loop, int interval, int amplitude)            | 开始播放效果@param loop 循环次数, 1不循环，大于1循环次数，-1无限循环；@param interval 循环间隔, 0-1000, 每次震动循环播放的间隔，单位ms;@param amplitude 振动强度，1-255，1最小，255最大。此参数用于修饰HE文件，进行整体的强度信号调整/缩减; |
| void start(int loop, int interval, int amplitude, int freq)  | 开始播放效果@param loop 循环次数, 1不循环，大于1循环次数，-1无限循环；@param interval 循环间隔, 0-1000, 每次震动循环播放的间隔，单位ms;@param amplitude 振动强度, 1-255，1最小，255最大。此参数用于修饰HE文件，进行整体的强度信号调整/缩减;@param freq 振动频率, 此参数用于修饰HE文件，进行整体的频率信号进行调整; |
| *void updateInterval(int interval)                           | 更新播放效果循环的时间间隔@param interval 循环间隔，interval取值区间[0,1000]，单位ms |
| *void updateAmplitude(int amplitude)                         | 更新播放效果的振动强度，对整体效果的强度信号进行放大、缩小@param amplitude 振动强度，amplitude取值区间[1,255] |
| *void updateFrequency(int freq)                              | 更新播放效果的频率，对整体效果的频率信号进行放大、缩小@param freq 振动频率 |
| *void updateParameter(int interval, int amplitude, int freq) | 更新播放效果相关参数interval\|amplitude == -1时，表示不更新@param interval 循环间隔@param amplitude 振动强度@param freq 振动频率, 预留 |
| public void stop()                                           | 停止播放振动效果                                             |

#### 调用示例

```
if (HapticPlayer.isAvailable()) {

DynamicEffect effect = DynamicEffect.create("{xxx: xxxx}");

HapticPlayer player = new HapticPlayer(effect);

player.start(0);

player.stop();

}
```

#### HE文件格式

```
{
  "Metadata": {
    "Version": 1,            // 版本号，整形
    "Created": "2020-07-08",       // 创建时间，String类型
    "Description": "game haptic"     // 震动效果描述，String类型
  },
  "Pattern": 
  [
    {
      "Event": {
        "Type": "continuous",  // 事件类型: continuous->持续震动。transient->简短震动
        "RelativeTime": 0,    // 相对开始时间, 整形, 单位ms
        "Duration": 300,     // 持续震动类型参数：持续时间。整形, 单位ms
        "Parameters": {
          "Intensity": 80,     // 震动强度, 整形, [0,100]。0->平台支持的最小值, 100->平台支持的最大值。
          "Frequency": 50,     // 震动频率, 整形, [0,100]。0->平台支持的最小值, 100->平台支持的最大值。
          "Curve": [        // 持续震动类型参数：曲线。实现上保证平滑过渡效果
            {"Time": 0，"Intensity": 0, "Frequency": 25}, // 起始点，必须。time为RelativeTime，Intensity必须取值为0。
            {"Time": 100,  "Intensity": 0.7,  "Frequency": -30}, 
            {"Time": 200,  "Intensity": 0.75,  "Frequency": 90},
            {"Time": 300，"Intensity": 0, "Frequency": 50} // 结束点，必须。time为Duration，Intensity必须取值为0。
          ]
        }
      }
    },
    {
      "Event": {
        "Type": "transient",     // 事件类型: continuous->持续震动, transient->简短震动
        "RelativeTime": 400,     // 相对开始时间, 整形, 单位ms
        "Parameters": {
          "Intensity": 80,     // 震动强度, 整形, [0,100]。0->平台支持的最小值, 100->平台支持的最大值。
          "Frequency": 40     // 震动频率, 整形, [0,100]。0->平台支持的最小值, 100->平台支持的最大值。
        }
      }
    }
  ]
}
```

详细说明如下：

1）He文件为json格式，包含“Metadata”字典和“Pattern”数组两部分内容；

2）Metadata字段包括振动效果的版本“Version”，创建时间“Created”，描述“Description”等基本信息；

3）具体的振动效果由“Pattern”字段描述，“Pattern”字段内容是一个“Event”数组，每个“Event”描述一个振动效果片段，不重叠；

4）“Event”效果片段

“Type”描述振动效果类型。包含两种类型：持续的振动类型”continuous”。简短振动类型” transient”。

“RelativeTime”，相对效果文件的开始时间，整形, 单位ms。

“Duration”，持续振动类型的持续时间，整形, 单位ms。

“Parameters”字段包括振动强度“Intensity”，振动频率“Frequency”。

“Curve”为效果曲线数组参数，用来描述持续振动的动态振动效果曲线，实现侧来实现动态变化效果的平滑过渡。其中起始点和结束点是必须的，都可调整频率值。中间为可选的控制点，可调频率及强度值。

“Curve“中Time为相对event的相对时间，Intensity对 Parameters中的Intensity进行修饰，取值范围为\[0,1\], 与Parameters中的Intensity相乘。Frequency对Parameters中的Frequency进行修饰，取值范围为\[-100,100\]，与Parameters中的Frequency进行相加。

# 四、接入指南

#### 准备开发环境

1.  为project/build.gradle添加maven仓库

```
示例：AGP版本7.2.1 ，gradle7.3.3 maven依赖配置（*注意AGP低版本在根目录中的build.gradle中配置该maven依赖）
maven {
    allowInsecureProtocol true
    url 'http://nexus.itgsa.com:5566/repository/release/'
    credentials {
        username 'developer'
        password 'developer!@#'
    }
}
```

1.  为app添加sdk, project/app/build.gradle

```
dependencies {
implementation ‘gsai.sdk:hestub:1.0.0'
}
```

1.  添加振动权限（小米侧需要）

```
project/app/src/main/AndroidManifest.xml
<uses-permission android:name="android.permission.VIBRATE"></uses-permission>
```



# 五、兼容性设计

使用implementation的构建方式将sdk代码嵌入项目中。若系统不存在HapticPlayer或DynamicEffect实现时，将由stubcode中的isAvailable返回false避免兼容性问题，在开发时应先进行判断后再使用：

```
if(HapticPlayer.isAvailable()) {
   do something your own
}
```

# 六、场景问题及处理

构建遇到以下错误：

FAILURE: Build failed with an exception.

\* What went wrong:

Could not determine the dependencies of task ':app:compileDebugJavaWithJavac'.

\> Could not resolve all task dependencies for configuration ':app:debugCompileClasspath'.

  \> Could not resolve gsai.sdk:roiencodestub:1.0.0-SNAPSHOT.

   Required by:

​     project :app

   \> Could not resolve gsai.sdk:roiencodestub:1.0.0-SNAPSHOT.

​     \> Unable to load Maven meta-data from https://nexus.itgsa.com:5566/repository/gsai-snapshot/gsai/sdk/roiencodestub/1.0.0-SNAPSHOT/maven-metadata.xml.

​      \> Could not get resource 'https://nexus.itgsa.com:5566/repository/gsai-snapshot/gsai/sdk/roiencodestub/1.0.0-SNAPSHOT/maven-metadata.xml'.

​        \> Could not GET 'https://nexus.itgsa.com:5566/repository/gsai-snapshot/gsai/sdk/roiencodestub/1.0.0-SNAPSHOT/maven-metadata.xml'.

​         \> Unsupported or unrecognized SSL message

网络问题，请更换网络环境。

# **联系我们**

service@itgsa.com